/*
 * This file is part of ELKI:
 * Environment for Developing KDD-Applications Supported by Index-Structures
 *
 * Copyright (C) 2022
 * ELKI Development Team
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
package elki.index.tree.spatial.rstarvariants.rdknn;

import elki.data.NumberVector;
import elki.distance.SpatialPrimitiveDistance;
import elki.index.tree.spatial.rstarvariants.RTreeSettings;

/**
 * Settings for the RdKNN Tree.
 * 
 * @author Erich Schubert
 * @since 0.7.0
 */
public class RdkNNSettings extends RTreeSettings {
  /**
   * Parameter k.
   */
  int k_max;

  /**
   * The distance function.
   */
  SpatialPrimitiveDistance<NumberVector> distance;

  /**
   * Constructor.
   *
   * @param k_max Maximum k to support
   * @param distance Distance function
   */
  public RdkNNSettings(int k_max, SpatialPrimitiveDistance<NumberVector> distance) {
    super();
    this.k_max = k_max;
    this.distance = distance;
  }
}
