/*
 * This file is part of ELKI:
 * Environment for Developing KDD-Applications Supported by Index-Structures
 *
 * Copyright (C) 2022
 * ELKI Development Team
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
package elki.outlier.distance;

import org.junit.Test;

import elki.data.DoubleVector;
import elki.database.Database;
import elki.outlier.AbstractOutlierAlgorithmTest;
import elki.result.outlier.OutlierResult;
import elki.utilities.ELKIBuilder;

/**
 * Tests the KNNWeightOutlier algorithm.
 *
 * @author Erich Schubert
 * @since 0.7.5
 */
public class HilOutTest extends AbstractOutlierAlgorithmTest {
  @Test
  public void testHilOut() {
    Database db = makeSimpleDatabase(UNITTEST + "outlier-3d-3clusters.ascii", 960);
    OutlierResult result = new ELKIBuilder<HilOut<DoubleVector>>(HilOut.class) //
        .with(HilOut.Par.K_ID, 4) //
        .with(HilOut.Par.N_ID, 200) //
        .build().autorun(db);
    assertAUC(db, "Noise", result, 0.985398148);
    assertSingleScore(result, 945, 1.70927657);
  }

  @Test
  public void testHilOutH16() {
    Database db = makeSimpleDatabase(UNITTEST + "outlier-3d-3clusters.ascii", 960);
    OutlierResult result = new ELKIBuilder<HilOut<DoubleVector>>(HilOut.class) //
        .with(HilOut.Par.K_ID, 4) //
        .with(HilOut.Par.N_ID, 200) //
        .with(HilOut.Par.H_ID, 16) //
        .build().autorun(db);
    assertAUC(db, "Noise", result, 0.985398148);
    assertSingleScore(result, 945, 1.70927657);
  }

  @Test
  public void testHilOutH8() {
    Database db = makeSimpleDatabase(UNITTEST + "outlier-3d-3clusters.ascii", 960);
    OutlierResult result = new ELKIBuilder<HilOut<DoubleVector>>(HilOut.class) //
        .with(HilOut.Par.K_ID, 4) //
        .with(HilOut.Par.N_ID, 200) //
        .with(HilOut.Par.H_ID, 8) //
        .build().autorun(db);
    assertAUC(db, "Noise", result, 0.985398148);
    assertSingleScore(result, 945, 1.70927657);
  }

  // Coarse enough to cause performance degradation:
  @Test
  public void testHilOutH2() {
    Database db = makeSimpleDatabase(UNITTEST + "outlier-3d-3clusters.ascii", 960);
    OutlierResult result = new ELKIBuilder<HilOut<DoubleVector>>(HilOut.class) //
        .with(HilOut.Par.K_ID, 4) //
        .with(HilOut.Par.N_ID, 200) //
        .with(HilOut.Par.H_ID, 2) //
        .build().autorun(db);
    assertAUC(db, "Noise", result, 0.8246);
    assertSingleScore(result, 945, 0.);
  }
}
